#include "Knight.h"

// Friend function to Calculate the new cell number for the move from the
// specified cell.  The new cell number is returned.
CELL_LOC CalcNewCell (CELL_LOC cellNum, MOVE moveTo);

// Constructor of an empty knight board
KNIGHT::KNIGHT()
	{  int i;
      for (i=0; i<(RowSize*ColSize); i++)
	  		Board[i] = 0;
      MarkedCellCnt = 0;
   }

// Reset will make the knight board empty
void KNIGHT::Reset()
	{  int i;
      for (i=0; i<(RowSize*ColSize); i++)
	  		Board[i] = 0;
      MarkedCellCnt = 0;
   }

// CheckCell will return true if the specified move from cellNum is OK and false
// if it is either not on the board or previously occupied. CheckCell will also
// return through the parameter list the CELL_LOC of the new location if it is
// OK to move there. If the move is not valid newCell is changed to -1.
// Precond:  cellNum must be a valid location on the board, moveTo must be
//				 between 1 and 8 inclusive. newCell must be a CELL_LOC variable.
// Postcond: If the specified move from cellNum is a valid empty cell newCell is
//				 changed to the destination of the move and the function returns
//				 true. Otherwise the function returns false and sets newCell to -1.
bool KNIGHT::CheckCell (CELL_LOC cellNum, MOVE moveTo, CELL_LOC & newCell)
   {  newCell = CalcNewCell(cellNum, moveTo);
      if ((newCell >= 0) && (newCell < (RowSize*ColSize)) && (!Board[newCell]))
	      return (true);
      newCell = -1;
      return (false);
   }

// PrintTofFile will print the board to the output file.  Cells which are
// marked will display a sequence number specifying when they were marked,
// otherwise a cell will be blank.
// Precond:  outFile must be open for output.
// Postcond: The board is printed to the output file.
void KNIGHT::PrintToFile (ofstream & outFile)
   {  int i, j;
      outFile << "\n ";
      for (j = 0; j < ColSize; j++)
	 		outFile << "+----";
      outFile << "+\n";
      for (i = 0; i < RowSize; i++)
	 		{  for (j = 0; j < ColSize; j++)
               {  if (Board[i*ColSize+j] == 0)
                     outFile << " |   ";
                  else if (Board[i*ColSize+j] <= 9)
                   	outFile << " |  " << Board[i*ColSize+j];
	       			else
                  	outFile << " | " << Board[i*ColSize+j];
               }
	    		outFile << " |\n ";
	    		for (j = 0; j < ColSize; j++)
	       		outFile << "+----";
	    		outFile << "+\n";
	 		}
   }

// PrintTofScreen will display the board on the screen.  Cells which are
// marked will display a sequence number specifying when they were marked,
// otherwise a cell will be blank.
// Precond:  None.
// Postcond: The board is printed to the screen.
void KNIGHT::PrintToScreen ()
   {  int i, j;
      cout << "\n ";
      for (j = 0; j < ColSize; j++)
	 		cout << "+----";
      cout << "+\n";
      for (i = 0; i < RowSize; i++)
	 		{  for (j = 0; j < ColSize; j++)
               {  if (Board[i*ColSize+j] == 0)
                   	cout << " |   " ;
                  else if (Board[i*ColSize+j] <= 9)
                   	cout << " |  " << Board[i*ColSize+j];
	       			else
                  	cout << " | " << Board[i*ColSize+j];
               }
	    		cout << " |\n ";
	    		for (j = 0; j < ColSize; j++)
	       		cout << "+----";
	    		cout << "+\n";
	 		}
   }

// MarkCell will mark the specified cell with the next sequence number.
// Precond:  newCell must be a valid cell on the board, which is not
// 			 currently marked.
// Postcond: newCell will hold the next sequence number of marked cells
// 			 and the next sequence number will be incremented.
void KNIGHT::MarkCell (CELL_LOC newCell)
   { if ((newCell >= 0) && (newCell < (RowSize*ColSize)))
   	if (!Board[newCell])
	    	{  MarkedCellCnt ++;
            Board[newCell] = MarkedCellCnt;
	      }
   }

// UnMarkCell will remove the sequence number from a cell and decrement
// the next sequence number.
// Precond:  currCell must be a valid cell on the board, and have the
// 			 value of the next sequence number.
// Postcond: currCell is returned to its initial unmarked value and the
//				 next sequence number is decremented.
void KNIGHT::UnMarkCell (CELL_LOC currCell)
  {  if ((currCell >= 0) && (currCell < (RowSize*ColSize)) &&
	 		(Board[currCell] == MarkedCellCnt))
			{  Board[currCell] = 0;
	   		MarkedCellCnt --;
			}
  }

// BoardFull will return true if the knight has visited every cell on
// the board and false otherwise.
// Precond:  self must exist.
// Postcond: The function will return true if the knight has visited
//				 every space on the board and false otherwise.
bool KNIGHT::BoardFull()
   {   if (MarkedCellCnt == (RowSize * ColSize))
       		return true;
       else return false;
   }

// Solve will return true if the knights's tour problem has a solution starting
// from the CurrCell.  It will store the solution in the Board. Solve is a
// recursive function.
// Precond:  CurrCell must be between 0 and (RowSize*ColSize - 1) inclusive.
// Postcond: Returns true if a solution exists starting from CurrCell, and false
//  			 otherwise.  The solution is stored in the Board.
bool KNIGHT::Solve(CELL_LOC CurrCell)
{
	CELL_LOC NewCell=0;
	MarkCell(CurrCell);
	if(BoardFull())
		return true;
	else 
	{
		for(MOVE i=1;i<=8;i++)
		{
			if(CheckCell(CurrCell,i,NewCell))
				if(Solve(NewCell)) return true;
		}
	}
	UnMarkCell(CurrCell);
	return false;
}

// Friend function to Calculate the new cell number for the move from the
// specified cell.  The new cell number is returned.
CELL_LOC CalcNewCell (CELL_LOC cellNum, MOVE moveTo)
   {  int row, col;
      CELL_LOC newCell= 0;
      row = cellNum/ColSize;
      col = cellNum%ColSize;
      if ((moveTo == 1) || (moveTo == 2))                		 /* Up 2 */
	   	row= row + 2;
      else if ((moveTo == 3) || (moveTo == 8))		   		 	 /* Up 1 */
	   	row = row + 1;
      else if ((moveTo == 4) || (moveTo == 7))           		 /* Down 1 */
	   	row = row - 1;
      else if ((moveTo == 5) || (moveTo == 6))                   /* Down 2 */
	   	row = row - 2;
      else newCell = -1;
      if ((row < 0) || (row >= RowSize))
         newCell = -1;

      if (newCell == 0)
	 		if ((moveTo == 1) || (moveTo == 6))				 	 /* Left 1 */
	      	col = col - 1;
	 		else if ((moveTo == 7) || (moveTo == 8))		     /* Left 2 */
	      	col = col - 2;
	 		else if ((moveTo == 2) || (moveTo == 5))			 /* Right 1 */
	      	col = col + 1;
	 		else if ((moveTo == 3) || (moveTo == 4)) 			 /* Right 2 */
	      	col = col + 2;
	 		else newCell = -1;
      if ((col<0)||(col >= ColSize))
      	newCell = -1;

      if (newCell == 0)
         newCell = row*ColSize + col;
      return (newCell);
   }

