#include <iostream>
#include <fstream>
using namespace std;

#define RowSize 3
#define ColSize 4


typedef int MOVE;
typedef int CELL_LOC;

#ifndef   _KNIGHT_
#define   _KNIGHT_

class KNIGHT
   {  public:
   		 // Constructor of an empty knight board
         KNIGHT();

		 // Reset will make the knight board empty
		 void Reset();

         // CheckCell will return true if the specified move from cellNum is OK
         // and false if it is either not on the board or previously occupied.
         // CheckCell will also return through the parameter list the CELL_LOC
         // of the new location if it is OK to move there. If the move is not
         // valid newCell is changed to -1.
         // Precond:  cellNum must be a valid location on the board, moveTo
         //				 must be between 1 and 8 inclusive. newCell must be a
         //				 CELL_LOC variable.
         // Postcond: If the specified move from cellNum is a valid empty cell
         //				 newCell is changed to the destination of the move and the
         //				 function returns true. Otherwise the function returns
         //				 false and sets newCell to -1.
	      bool CheckCell (CELL_LOC cellNum, MOVE moveTo, CELL_LOC &newCell);

         // PrintTofFile will print the board to the output file.  Cells which
         // are marked will display a sequence number specifying when they were
         // marked, otherwise a cell will be blank.
         // Precond:  outFile must be open for output.
         // Postcond: The board is printed to the output file.
			void PrintToFile (ofstream & outFile);

         // PrintTofScreen will display the board on the screen.  Cells which
         // are marked will display a sequence number specifying when they were
         // marked, otherwise a cell will be blank.
         // Precond:  None.
         // Postcond: The board is printed to the screen.
         void PrintToScreen ();

         // MarkCell will mark the specified cell with the next sequence number.
         // Precond:  newCell must be a valid cell on the board, which is not
         // 			 currently marked.
         // Postcond: newCell will hold the next sequence number of marked cells
         // 			 and the next sequence number will be incremented.
			void MarkCell (CELL_LOC newCell);

         // UnMarkCell will remove the sequence number from a cell and decrement
         // the next sequence number.
         // Precond:  currCell must be a valid cell on the board, and have the
         // 			 value of the next sequence number.
         // Postcond: currCell is returned to its initial unmarked value and the
         //				 next sequence number is decremented.
			void UnMarkCell (CELL_LOC currcell);

		 // BoardFull will return true if the knight has visited every cell on
         // the board and false otherwise.
         // Precond:  self must exist.
         // Postcond: The function will return true if the knight has visited
         //				 every space on the board and false otherwise.
         bool BoardFull();

         // Solve will return true if the knights's tour problem has a solution
         // starting from the CurrCell.  It will store the solution in the
         // Board. Solve is a recursive function.
		 // Precond:  CurrCell must be between 0 and (RowSize*ColSize - 1)
         // 			 inclusive.
		 // Postcond: Returns true if a solution exists starting from CurrCell,
         //				 and false otherwise.  The solution is stored in the Board.
			bool Solve(CELL_LOC CurrCell);

   	private:
      	int Board[RowSize*ColSize];         // An array of 12 locations
         									// corresponding to the locations
                                            // on a 3x4 board.  Each location
                                            // contains an integer specifying
                                            // when that cell was visited.
         int MarkedCellCnt;					// An integer between 0 and 12
};

#endif;

